"""
Experimental: reusable invoke tasks.
"""
import re
from pathlib import Path

from invoke import Context

TASKS = ["help", "help_make"]


def help(c: Context):
    c.run("invoke --list")


def help_make(c: Context):
    """Helper to generate the `make help` message"""
    # TODO: should try harder to find the right file
    with Path("Makefile").open() as f:
        makefile = f.read()

    target = ""
    description = ""
    targets = []
    for line in makefile.splitlines():
        if m := re.match(r"^## (.*)", line):
            description = m.group(1)
        elif m := re.match("^(.*?):", line):
            target = m.group(1)
            if description:
                targets.append([target, description])
        else:
            target = ""
            description = ""

    max_len = max(len(t[0]) for t in targets)

    print("Targets:\n")
    for targets, description in targets:
        print(f"{targets:<{max_len}}  {description}")



"""Invoke tasks for subrepos.

Should be used at a higher level, e.g. for constructing "subrepos" tasks.
"""
from collections.abc import Sequence

from invoke import Context


def h1(msg: str):
    print(str)


def run_in_subrepos(c: Context, cmd: str, sub_repos: Sequence[str] = ()):
    for sub_repo in sub_repos:
        h1(f"Running '{cmd}' in subrepos: {sub_repo}")
        with c.cd(sub_repo):
            c.run(cmd, echo=True)



"""
Experimental: reusable invoke tasks.
"""

from invoke import task
from . import help, versions


def import_tasks(namespace):
    """Import tasks from other modules."""
    for module in [help, versions]:
        for task_name in module.TASKS:
            _task = getattr(module, task_name)
            namespace[_task.__name__] = task(_task)



"""Version bumpers, using date-based versioning.

Todo: also support semantic versionning.
"""

import sys
from time import strftime, gmtime

import tomlkit
from invoke import Context

TASKS = ["bump_version"]

def bump_version(c: Context):
    """Bump version in pyproject.toml, commit & apply tag"""

    r = c.run("git diff --quiet", echo=True, warn=True)
    if r.exited != 0:
        print("git diff --quiet failed")
        sys.exit()

    update_version()
    c.run(f"git tag {get_version()}", echo=True)
    c.run("git add pyproject.toml", echo=True)
    c.run("git commit -m 'Bump version'", echo=True)


def update_version():
    pyproject_d = tomlkit.parse(open("pyproject.toml").read())
    version = pyproject_d["tool"]["poetry"]["version"]
    serial = int(version.split(".")[-1])
    date = version[: -len(str(serial)) - 1]

    new_date = strftime("%Y.%m.%d", gmtime())
    if date == new_date:
        serial += 1
    else:
        serial = 1

    new_version = f"{new_date}.{serial}"
    print("New version: ", new_version)
    pyproject_d["tool"]["poetry"]["version"] = new_version
    with open("pyproject.toml", "w") as f:
        f.write(tomlkit.dumps(pyproject_d))


def get_version():
    d = tomlkit.parse(open("pyproject.toml").read())
    version = d["tool"]["poetry"]["version"]
    return version
